@@ -251,6 +251,8 @@ struct {
 	int	flags;
 } compat_tab[] =
 {
+	/* APC600 */
+	{ "6QD",	"79ABCDEFGKLMNOPQRSUVWXYZcefgjklmnopqrsuxz", 0 },
 	/* CS 350 */
 	{ "5.4.D",	"\1ABPQRSUYbdfgjmnx9",	0 },
 
