@@ -42,7 +42,7 @@ typedef int bool;
 #define MODE_OPEN 0
 #define MODE_REOPEN 1
 
-#define MAX_TS			2		/* validity period of a gotten report (1 sec) */
+#define MAX_TS			2		/* validity period of a gotten report (2 sec) */
 
 /* ---------------------------------------------------------------------- */
 
@@ -58,6 +58,7 @@ typedef struct
 	u_int16_t ProductID; /*!< Device's Product ID */
 	int       Application; /*!< match Usage for HIDOpenDevice(Usage)) */
 	char*     Serial; /* Product serial number */
+	u_char    iProduct;
 	int       fd; /* "internal" file descriptor */
 } HIDDevice;
 
@@ -74,7 +75,7 @@ typedef struct
 	long    LogMin;		/*!< Logical Min							*/
 	long    LogMax;		/*!< Logical Max						*/
 	long    PhyMin;		/*!< Physical Min						*/
-	long    PhyMax;		/*!< Physical Max						*/
+	long    PhyMax;		/*!< Physical Max						*/	
 } HIDItem;
 
 /*!
@@ -105,22 +106,22 @@ HIDItem *HIDGetItem(const char *ItemPath);
 /*
  * HIDGetItemValue
  * -------------------------------------------------------------------------- */
-float HIDGetItemValue(const char *path, float *Value);
+float HIDGetItemValue(char *path, float *Value);
 
 /*
  * HIDGetItemString
  * -------------------------------------------------------------------------- */
-char *HIDGetItemString(const char *path);
+char *HIDGetItemString(char *path);
 
 /*
  * HIDSetItemValue
  * -------------------------------------------------------------------------- */
-bool HIDSetItemValue(const char *path, float value);
+bool HIDSetItemValue(char *path, float value);
 
 /*
  * HIDGetNextEvent
  * -------------------------------------------------------------------------- */
-HIDItem *HIDGetNextEvent(HIDDevice *dev);
+int HIDGetEvents(HIDDevice *dev, HIDItem **eventsList);
 
 /*
  * HIDCloseDevice
@@ -130,7 +131,5 @@ void HIDCloseDevice(HIDDevice *dev);
 
 
 void HIDDumpTree(HIDDevice *hd);
-int hid_lookup_usage(char *name);
-
 
 #endif /* _LIBHID_H */
