@@ -1,13 +1,13 @@
 #include "common.h"
-#include "upscommon.h"
 #include "upsconf.h"
 #include "dstate.h"
 #include "extstate.h"
 
 /* public functions & variables from main.c */
 extern const char *progname;
+extern char *device_path;
 extern const char *device_name;
-extern const char *device_path;
+extern int	upsfd, extrafd, broken_driver, experimental_driver, exit_flag;
 
 /* functions & variables required in each driver */
 void upsdrv_initups(void);	/* open connection to UPS, fail if not found */
@@ -47,5 +47,3 @@ typedef struct {
 
 /* callback from driver - create the table for future -x entries */
 void addvar(int vartype, const char *name, const char *desc);
-
-	extern	int	broken_driver;
