@@ -1,6 +1,6 @@
 /*  mge-shut.c - monitor MGE UPS for NUT with SHUT protocol
  * 
- *  Copyright (C) 2002-2004
+ *  Copyright (C) 2002 - 2005
  *     Arnaud Quette <arnaud.quette@free.fr> & <arnaud.quette@mgeups.com>
  *     Philippe Marzouk <philm@users.sourceforge.net>
  *     Russell Kroll <rkroll@exploits.org>
@@ -81,8 +81,8 @@ u_char 			raw_buf[4096];
 /* --------------------------------------------------------------- */
 
 float expo(int a, int b);
-void format_model_name(char *iProduct, char *iModel);
 extern long FormatValue(long Value, u_char Size);
+static char *hu_find_infoval(info_lkp_t *hid2info, long value);
 
 /* --------------------------------------------------------------- */
 /*                    UPS Driver Functions                         */
@@ -98,35 +98,44 @@ void upsdrv_initinfo (void)
 	shut_identify_ups ();
 
 	printf("Detected %s [%s] on %s\n", dstate_getinfo("ups.model"),
-		   dstate_getinfo("ups.serial"), device_path);
-
+			dstate_getinfo("ups.serial"), device_path);
 
 	/* Device capabilities enumeration ----------------------------- */
 	for ( item = mge_info ; item->type != NULL ; item++ ) {
 		
+		/* Check if we are asked to stop (reactivity++) */
+		if (exit_flag != 0)
+		  return;
+
 		/* avoid redundancy when multiple defines (RO/RW) */
 		if (dstate_getinfo(item->type) != NULL)
 			continue;
 		
 		/* Special case for handling server side variables */
 		if (item->shut_flags & SHUT_FLAG_ABSENT) {
-		  /* Check if exists (if necessary) before creation */
-		  if ((item->item_path != NULL) &&
-			  (hid_get_value(item->item_path) != 1 ) )
+			/* Check if exists (if necessary) before creation */
+			if (item->item_path != NULL)
+			  {
+				if (hid_get_value(item->item_path) != 1 )
+				  continue;
+			  }
+			else
 			{
+			  /* Simply set the default value */
+			  dstate_setinfo(item->type, "%s", item->dfl);
+			  dstate_setflags(item->type, item->flags);
 			  continue;
 			}
+			
+			dstate_setinfo(item->type, "%s", item->dfl);
+			dstate_setflags(item->type, item->flags);
 
-		  dstate_setinfo(item->type, "%s", item->dfl);
-		  dstate_setflags(item->type, item->flags);;
-		  
-		  /* Set max length for strings, if neede	d */
-		  if (item->flags & ST_FLAG_STRING)
-			dstate_setaux(item->type, item->length);
-
-		  /* disable reading now => needed?!
-			 item->shut_flags &= ~HU_FLAG_OK;*/
-
+			/* Set max length for strings, if needed */
+			if (item->flags & ST_FLAG_STRING)
+				dstate_setaux(item->type, item->length);
+			
+			/* disable reading now 
+			item->shut_flags &= ~SHUT_FLAG_OK;*/
 		} else {
 			if (hid_get_value(item->item_path) != 0 ) {
 
@@ -162,6 +171,7 @@ void upsdrv_initinfo (void)
 void upsdrv_updateinfo (void)
 {
 	mge_info_item *item;
+	char *nutvalue;
 	
 	upsdebugx(2, "entering upsdrv_updateinfo()");
 	
@@ -180,6 +190,10 @@ void upsdrv_updateinfo (void)
 	/* Device data walk ----------------------------- */
 	for ( item = mge_info ; item->type != NULL; item++ ) {
 
+	  /* Check if we are asked to stop (reactivity++) */
+	  if (exit_flag != 0)
+		return;
+
     if (item->shut_flags & SHUT_FLAG_ABSENT)
 			continue;
 
@@ -189,7 +203,18 @@ void upsdrv_updateinfo (void)
 				upsdebugx(3, "%s: hData.Value = %ld (%ld)",
 					item->item_path, hData.Value, hData.LogMax);
 				
-				dstate_setinfo(item->type, item->fmt, hData.Value);
+				/* need lookup'ed translation */
+				if (item->hid2info != NULL)
+				  {
+					nutvalue = hu_find_infoval(item->hid2info, (long)hData.Value);
+					if (nutvalue != NULL)
+					  dstate_setinfo(item->type, "%s", nutvalue);
+					else
+					  dstate_setinfo(item->type, item->fmt, hData.Value);
+				  }
+				else
+				  dstate_setinfo(item->type, item->fmt, hData.Value);
+
 				dstate_dataok();
 			} else {
 				if (shut_ups_start () != 0)
@@ -411,6 +436,7 @@ int shut_identify_ups ()
 {
 	char string[MAX_STRING];
 	char model[MAX_STRING];
+	char *finalname = NULL;
 	int retcode;
 	
 	if (commstatus == 0)
@@ -425,14 +451,24 @@ int shut_identify_ups ()
 		
 		strcpy(model, string);
 		
-		if(hid_get_value("UPS.PowerSummary.iModel") != 0 ) {
+		if(hid_get_value("UPS.PowerSummary.iModel") != 0 )
+		  {
 			if((shut_get_string(hData.Value, string, 0x25)) > 0)
-				format_model_name(model, string);
-		}
+			  finalname = get_model_name(model, string);
+		  }
 		else
-			format_model_name(model, NULL);
+		  {
+			/* Try with "UPS.Flow.[4].ConfigApparentPower" */
+			if(hid_get_value("UPS.Flow.[4].ConfigApparentPower") != 0 )
+			  {
+				sprintf(&string[0], "%i", (int)hData.Value);
+				finalname = get_model_name(model, string);
+			  }
+			else
+			  finalname = get_model_name(model, NULL);
+		  }
 
-		dstate_setinfo("ups.model", "%s", model);
+		dstate_setinfo("ups.model", "%s", finalname);
 	}
 		
 	/* Get strings iSerialNumber */
@@ -444,7 +480,8 @@ int shut_identify_ups ()
 	else
 		dstate_setinfo("ups.serial", "unknown");
 
-	return 0;
+	/* all went fine */
+	return 1;
 }
 
 /**********************************************************************
@@ -636,6 +673,24 @@ void  shut_ups_status(void)
 		if(hData.Value == 1)
 			status_set("RB");
 	}
+  
+	if(hid_get_value("UPS.PowerSummary.PresentStatus.Good") != 0 ) {
+		if(hData.Value == 0)
+			status_set("OFF");
+	}
+
+	/* FIXME: extend ups.status for BYPASS: */
+	/* Manual bypass */
+	if(hid_get_value("UPS.PowerConverter.Input.[4].PresentStatus.Used") != 0 ) {
+		if(hData.Value == 1)
+			status_set("BYPASS");
+	}
+	/* Automatic bypass */
+	if(hid_get_value("UPS.PowerConverter.Input.[2].PresentStatus.Used") != 0 ) {
+		if(hData.Value == 1)
+			status_set("BYPASS");
+	}
+
 	status_commit();
 }
 
@@ -1171,8 +1226,7 @@ int hid_get_value(const char *item_path)
 		return 0;
 	}
 
-	/* all went fine */
-	return 1;
+	return 0;
 }
 
   /* 
@@ -1291,50 +1345,28 @@ float expo(int a, int b)
 	return -1;
 }
 
-/*  Formatmodel names */
-void format_model_name(char *iProduct, char *iModel)
+/*  Format model names */
+char *get_model_name(char *iProduct, char *iModel)
 {
-	/* Evolution model range */
-	if(!strncmp(iProduct, "Evolution", 9)) {
-		if(iModel != NULL)
-			sprintf(iProduct, "evolution %d", atoi(iModel));
-			
-		return;
+  models_name_t *model = NULL;
+
+  upsdebugx(2, "get_model_name(%s, %s)\n", iProduct, iModel);
+
+  /* Search for formatting rules */
+  for ( model = models_names ; model->iProduct != NULL ; model++ )
+	{
+	  upsdebugx(2, "comparing with: %s", model->finalname);
+	  if ( (!strncmp(iProduct, model->iProduct, strlen(model->iProduct)))
+		   && (!strncmp(iModel, model->iModel, strlen(model->iModel))) )
+		{
+		  upsdebugx(2, "Found %s\n", model->finalname);
+		  break;
+		}
 	}
-
-	/* Ellipse Premium model range */
-	if(!strncmp(iProduct, "ellipse", 7)) {
-		if(iModel != NULL)
-			/* Note: we skip "PRxxxx" for the value */
-			sprintf(iProduct, "ellipse premium %d", atoi(iModel+2));
-		else
-			sprintf(iProduct, "ellipse premium");
-		
-		return;
-	}
-
-	/* Ellipse model range */
-	if(!strncmp(iProduct, "ELLIPSE", 7)) {
-		/* This gives 500 for Ellipse 500 ... */
-		if(hid_get_value("UPS.Flow.[4].ConfigApparentPower") != 0 )
-			sprintf(iProduct, "ellipse %ld", hData.Value);
-		else
-			sprintf(iProduct, "ellipse");
-
-		return;
-	}
-
-	/* NOVA model range */
-	if(!strncmp(iProduct, "NOVA", 4)) {
-		/* This gives 600 for NOVA 600 AVR... */
-		if(hid_get_value("UPS.Flow.[4].ConfigApparentPower") != 0 )
-			sprintf(iProduct, "NOVA %ld AVR", hData.Value);
-		else
-			sprintf(iProduct, "NOVA AVR");
-
-		return;
-	}
-
+  /* FIXME: if we end up with model->iProduct == NULL
+   * then process name in a generic way (not yet supported models!)
+   */
+  return model->finalname;
 }
 
 /* set r/w INFO_ element to a value. */
@@ -1414,7 +1446,9 @@ int hid_set_value(const char *varname, const char *val)
 					upsdebugx(3, "FAILED");
 				*/				
 				return STAT_SET_HANDLED;
-			}			
+			}
+			else
+				upsdebugx(3, "Object is constant");
 		}
 		else
 			upsdebugx(3, "Can't find object");
@@ -1437,3 +1471,24 @@ mge_info_item *shut_find_info(const char *varname)
 	fatalx("shut_find_info: unknown info type: %s", varname);
 	return NULL;
 }
+
+/* find the NUT value matching that HID Item value */
+static char *hu_find_infoval(info_lkp_t *hid2info, long value)
+{
+  info_lkp_t *info_lkp;
+  
+  upsdebugx(3, "hu_find_infoval: searching for value = %ld\n", value);
+  
+  for (info_lkp = hid2info; (info_lkp != NULL) &&
+	 (strcmp(info_lkp->nut_value, "NULL")); info_lkp++) {
+    
+    if (info_lkp->hid_value == value) {
+      upsdebugx(3, "hu_find_infoval: found %s (value: %ld)\n",
+		info_lkp->nut_value, value);
+      
+      return info_lkp->nut_value;
+    }
+  }
+  upsdebugx(3, "hu_find_infoval: no matching INFO_* value for this HID value (%ld)\n", value);
+  return NULL;
+}
