@@ -173,7 +173,8 @@ static void shutdown_return(int delay)
 
 	if ((delay < 1) || (delay > 999))
 	{
-		upslogx(LOG_ERR, "Shutdown delay %d is not within valid range [1..999]");
+		upslogx(LOG_ERR, "Shutdown delay %d is not within valid range [1..999]",
+			delay);
 		return;
 	}
 
